@@ -49,7 +49,18 @@ enum mailbox_flags {
 	/* Force opening mailbox and ignoring any ACLs */
 	MAILBOX_FLAG_IGNORE_ACLS	= 0x100,
 	/* Open mailbox even if it's already marked as deleted */
-	MAILBOX_FLAG_OPEN_DELETED	= 0x200
+	MAILBOX_FLAG_OPEN_DELETED	= 0x200,
+	/* Mailbox is opened for deletion, which should be performed as
+	   efficiently as possible, even allowing the mailbox state to become
+	   inconsistent. For example this disables lazy_expunge plugin and
+	   quota updates (possibly resulting in broken quota). and This is
+	   useful for example when deleting entire user accounts. */
+	MAILBOX_FLAG_DELETE_UNSAFE	= 0x400,
+	/* Mailbox is used for caching purposes. Some of the mails may be
+	   stubs, which exist in the index but that don't have a mail body.
+	   The backend shouldn't treat it as corruption if a mail body isn't
+	   found. */
+	MAILBOX_FLAG_USE_STUBS		= 0x800,
 };
 
 enum mailbox_feature {
@@ -91,10 +102,12 @@ enum mailbox_metadata_items {
 	MAILBOX_METADATA_CACHE_FIELDS		= 0x04,
 	MAILBOX_METADATA_PRECACHE_FIELDS	= 0x08,
 	MAILBOX_METADATA_BACKEND_NAMESPACE	= 0x10,
-	MAILBOX_METADATA_PHYSICAL_SIZE		= 0x20
+	MAILBOX_METADATA_PHYSICAL_SIZE		= 0x20,
+	MAILBOX_METADATA_FIRST_SAVE_DATE	= 0x40
 	/* metadata items that require mailbox to be synced at least once. */
 #define MAILBOX_METADATA_SYNC_ITEMS \
-	(MAILBOX_METADATA_VIRTUAL_SIZE | MAILBOX_METADATA_PHYSICAL_SIZE)
+	(MAILBOX_METADATA_VIRTUAL_SIZE | MAILBOX_METADATA_PHYSICAL_SIZE | \
+	 MAILBOX_METADATA_FIRST_SAVE_DATE)
 };
 
 enum mailbox_search_result_flags {
@@ -177,7 +190,15 @@ enum mailbox_transaction_flags {
 	MAILBOX_TRANSACTION_FLAG_NO_CACHE_DEC	= 0x10,
 	/* Sync transaction describes changes to mailbox that already happened
 	   to another mailbox with whom we're syncing with (dsync) */
-	MAILBOX_TRANSACTION_FLAG_SYNC		= 0x20
+	MAILBOX_TRANSACTION_FLAG_SYNC		= 0x20,
+	/* Don't trigger any notifications for this transaction. This
+	   especially means the notify plugin. This would normally be used only
+	   with _FLAG_SYNC. */
+	MAILBOX_TRANSACTION_FLAG_NO_NOTIFY	= 0x40,
+	/* Append fills in an existing stub mail for the specified UID,
+	   instead of saving a new mail. This requires mailbox to be opened
+	   with MAILBOX_FLAG_USE_STUBS. */
+	MAILBOX_TRANSACTION_FLAG_FILL_IN_STUB	= 0x80,
 };
 
 enum mailbox_sync_flags {
@@ -238,6 +259,8 @@ struct mailbox_status {
 
 	/* These flags can be permanently modified (STATUS_PERMANENT_FLAGS) */
 	enum mail_flags permanent_flags;
+	/* These flags can be modified (STATUS_PERMANENT_FLAGS) */
+	enum mail_flags flags;
 
 	/* All keywords can be permanently modified (STATUS_PERMANENT_FLAGS) */
 	unsigned int permanent_keywords:1;
@@ -271,10 +294,15 @@ struct mailbox_metadata {
 	uint64_t virtual_size;
 	/* sum of physical size of all messages in mailbox */
 	uint64_t physical_size;
+	/* timestamp of when the first message was saved.
+	   (time_t)-1 if there are no mails in the mailbox. */
+	time_t first_save_date;
+
 	/* Fields that have "temp" or "yes" caching decision. */
 	const ARRAY_TYPE(mailbox_cache_field) *cache_fields;
 	/* Fields that should be precached */
 	enum mail_fetch_field precache_fields;
+
 	/* imapc backend returns this based on the remote NAMESPACE reply,
 	   while currently other backends return "" and type the same as the
 	   mailbox's real namespace type */
@@ -336,12 +364,23 @@ ARRAY_DEFINE_TYPE(mailbox_expunge_rec, struct mailbox_expunge_rec);
 enum mail_lookup_abort {
 	/* Perform everything no matter what it takes */
 	MAIL_LOOKUP_ABORT_NEVER = 0,
-	/* Abort if the operation would require reading message header/body */
+	/* Abort if the operation would require reading message header/body or
+	   otherwise opening the mail file (e.g. with dbox metadata is read by
+	   opening and reading the file). This still allows somewhat fast
+	   operations to be performed, such as stat()ing a file. */
 	MAIL_LOOKUP_ABORT_READ_MAIL,
 	/* Abort if the operation can't be done fully using cache file */
 	MAIL_LOOKUP_ABORT_NOT_IN_CACHE
 };
 
+enum mail_access_type {
+	MAIL_ACCESS_TYPE_DEFAULT = 0,
+	/* Mail is being used for searching */
+	MAIL_ACCESS_TYPE_SEARCH,
+	/* Mail is being used for sorting results */
+	MAIL_ACCESS_TYPE_SORT,
+};
+
 struct mail {
 	/* always set */
 	struct mailbox *box;
@@ -353,6 +392,18 @@ struct mail {
 	unsigned int has_nuls:1; /* message data is known to contain NULs */
 	unsigned int has_no_nuls:1; /* -''- known to not contain NULs */
 
+	/* Mail's header/body stream was opened within this request.
+	   If lookup_abort!=MAIL_LOOKUP_ABORT_NEVER, this can't become TRUE. */
+	bool mail_stream_opened:1;
+	/* Mail's fast metadata was accessed within this request, e.g. the mail
+	   file was stat()ed. If mail_stream_opened==TRUE, this value isn't
+	   accurate anymore, because some backends may always set this when
+	   stream is opened and some don't. If lookup_abort is
+	   MAIL_LOOKUP_ABORT_NOT_IN_CACHE, this can't become TRUE. */
+	bool mail_metadata_accessed:1;
+
+	enum mail_access_type access_type;
+
 	/* If the lookup is aborted, error is set to MAIL_ERROR_NOTPOSSIBLE */
 	enum mail_lookup_abort lookup_abort;
 };
@@ -470,6 +521,11 @@ bool mailbox_equals(const struct mailbox *box1,
 /* Returns TRUE if the mailbox is user's INBOX or another user's shared INBOX */
 bool mailbox_is_any_inbox(struct mailbox *box);
 
+/* Change mailbox_verify_create_name() to not verify new mailbox name
+   restrictions (but still check that it's a valid existing name). This is
+   mainly used by dsync to make sure the sync works even though the original
+   name isn't valid anymore. */
+void mailbox_skip_create_name_restrictions(struct mailbox *box, bool set);
 /* Returns -1 if mailbox_create() is guaranteed to fail because the mailbox
    name is invalid, 0 not. The error message contains a reason. */
 int mailbox_verify_create_name(struct mailbox *box);
@@ -723,12 +779,11 @@ void mailbox_save_set_pop3_uidl(struct mail_save_context *ctx,
    of the mailbox. Not all backends support this. */
 void mailbox_save_set_pop3_order(struct mail_save_context *ctx,
 				 unsigned int order);
-/* If dest_mail is set, the saved message can be accessed using it. Note that
-   setting it may require mailbox syncing, so don't set it unless you need
-   it. Also you shouldn't try to access it before mailbox_save_finish() is
-   called. */
+/* FIXME: Remove in v2.3. Obsolete - use mailbox_save_get_dest_mail() instead */
 void mailbox_save_set_dest_mail(struct mail_save_context *ctx,
 				struct mail *mail);
+/* Returns the destination mail */
+struct mail *mailbox_save_get_dest_mail(struct mail_save_context *ctx);
 /* Begin saving the message. All mail_save_set_*() calls must have been called
    before this function. If the save initialization fails, the context is freed
    and -1 is returned. After beginning the save you should keep calling
@@ -814,7 +869,7 @@ int mail_get_first_header(struct mail *mail, const char *field,
 int mail_get_first_header_utf8(struct mail *mail, const char *field,
 			       const char **value_r);
 /* Return a NULL-terminated list of values for each found field.
-   Returns -1 if error, 0 otherwise (with or without headers found). */
+   Returns 1 if headers were found, 0 if not (value_r==NULL) or -1 if error. */
 int mail_get_headers(struct mail *mail, const char *field,
 		     const char *const **value_r);
 /* Like mail_get_headers(), but decode MIME encoded words to UTF-8.
@@ -835,10 +890,21 @@ int mail_get_header_stream(struct mail *mail,
 int mail_get_stream(struct mail *mail, struct message_size *hdr_size,
 		    struct message_size *body_size, struct istream **stream_r)
 	ATTR_NULL(2, 3);
+/* Same as mail_get_stream(), but specify a reason why the mail is being read.
+   This can be useful for debugging purposes. */
+int mail_get_stream_because(struct mail *mail, struct message_size *hdr_size,
+			    struct message_size *body_size,
+			    const char *reason, struct istream **stream_r)
+	ATTR_NULL(2, 3);
 /* Similar to mail_get_stream(), but the stream may or may not contain the
    message body. */
 int mail_get_hdr_stream(struct mail *mail, struct message_size *hdr_size,
 			struct istream **stream_r) ATTR_NULL(2);
+/* Same as mail_get_hdr_stream(), but specify a reason why the header is being
+   read. This can be useful for debugging purposes. */
+int mail_get_hdr_stream_because(struct mail *mail,
+				struct message_size *hdr_size,
+				const char *reason, struct istream **stream_r);
 /* Returns the message part's body decoded to 8bit binary. If the
    Content-Transfer-Encoding isn't supported, returns -1 and sets error to
    MAIL_ERROR_CONVERSION. If the part refers to a multipart, all of its
@@ -881,15 +947,25 @@ void mail_expunge(struct mail *mail);
 void mail_precache(struct mail *mail);
 /* Mark a cached field corrupted and have it recalculated. */
 void mail_set_cache_corrupted(struct mail *mail, enum mail_fetch_field field);
+void mail_set_cache_corrupted_reason(struct mail *mail,
+				     enum mail_fetch_field field,
+				     const char *reason);
 
 /* Return 128 bit GUID using input string. If guid is already 128 bit hex
    encoded, it's returned as-is. Otherwise SHA1 sum is taken and its last
    128 bits are returned. */
 void mail_generate_guid_128_hash(const char *guid, guid_128_t guid_128_r);
 
-/* Parse a human-writable string into a timestamp. Returns 0 and timestamp on
-   success, -1 if the string couldn't be parsed. Currently supported string
-   formats: yyyy-mm-dd, imap date, unix timestamp, interval (e.g. n days). */
-int mail_parse_human_timestamp(const char *str, time_t *timestamp_r);
+/* Parse a human-writable string into a timestamp. utc_r controls whether
+   the returned timestamp should be treated as an exact UTC time (TRUE), or
+   whether this is a human-given date where the timestamp could be adjusted
+   by the matched mails' timezones (see MAIL_SEARCH_ARG_FLAG_USE_TZ).
+
+   Returns 0 and timestamp on success, -1 if the string couldn't be parsed.
+   Currently supported string formats: yyyy-mm-dd (utc=FALSE),
+   imap date (utc=FALSE), unix timestamp (utc=TRUE), interval (e.g. n days,
+   utc=TRUE). */
+int mail_parse_human_timestamp(const char *str, time_t *timestamp_r,
+			       bool *utc_r);
 
 #endif
