@@ -37,6 +37,9 @@ struct mail_user {
 	struct ip_addr *local_ip, *remote_ip;
 	const char *auth_token, *auth_user;
 	const char *const *userdb_fields;
+	/* Timestamp when this session was initially created. Most importantly
+	   this stays the same after IMAP client is hibernated and restored. */
+	time_t session_create_time;
 
 	const struct var_expand_table *var_expand_table;
 	/* If non-NULL, fail the user initialization with this error.
@@ -69,6 +72,9 @@ struct mail_user {
 	unsigned int autocreated:1;
 	/* mail_user_init() has been called */
 	unsigned int initialized:1;
+	/* The initial namespaces have been created and
+	   hook_mail_namespaces_created() has been called. */
+	unsigned int namespaces_created:1;
 	/* SET_STR_VARS in user's all settings have been expanded.
 	   This happens near the beginning of the user initialization,
 	   so this is rarely needed to be checked. */
@@ -91,6 +97,8 @@ struct mail_user {
 	unsigned int stats_enabled:1;
 	/* Enable autoexpunging at deinit. */
 	unsigned int autoexpunge_enabled:1;
+	/* This session was restored (e.g. IMAP unhibernation) */
+	unsigned int session_restored:1;
 };
 
 struct mail_user_module_register {
@@ -108,6 +116,10 @@ extern const struct var_expand_func_table *mail_user_var_expand_func_table;
 struct mail_user *mail_user_alloc(const char *username,
 				  const struct setting_parser_info *set_info,
 				  const struct mail_user_settings *set);
+struct mail_user *
+mail_user_alloc_nodup_set(const char *username,
+			  const struct setting_parser_info *set_info,
+			  const struct mail_user_settings *set);
 /* Returns -1 if settings were invalid. */
 int mail_user_init(struct mail_user *user, const char **error_r);
 
@@ -168,6 +180,10 @@ const char *mail_user_get_anvil_userip_ident(struct mail_user *user);
 struct mail_storage *
 mail_user_get_storage_class(struct mail_user *user, const char *name);
 
+/* Initialize SSL client settings from mail_user settings. */
+void mail_user_init_ssl_client_settings(struct mail_user *user,
+				struct ssl_iostream_settings *ssl_set);
+
 /* Initialize fs_settings from mail_user settings. */
 void mail_user_init_fs_settings(struct mail_user *user,
 				struct fs_settings *fs_set,
